import os
from zipfile import ZipFile
from django.template.loader import render_to_string
from django.template.loader import TemplateDoesNotExist

import six


class TemplateZipFile(ZipFile, object):
    """
    Usage::

        from zipfile import ZIP_DEFLATED
        from django_zipfile import TemplateZipFile

        def myview(request, object_id):
            obj = get_object_or_404(MyModel, pk=object_id)
            context = {
                'object': obj
            }
            response = HttpResponse(mimetype='application/octet-stream')
            response['Content-Disposition'] = 'attachment; filename=myfile.zip'
            container = TemplateZipFile(response, mode='w', compression=ZIP_DEFLATED, template_root='myapp/myzipskeleton/')

            container.write_template('mimetype')
            container.write_template('META-INF/container.xml')
            container.write_template('chapter1.html', context=context)

            container.close()
            return response
    """

    def __init__(self, file, template_root=None, *args, **kwargs):
        self.template_root = []
        for root in self._to_list(template_root):
            if not root.endswith('/'):
                root += '/'
            self.template_root.append(root)
        super(TemplateZipFile, self).__init__(file, *args, **kwargs)

    def _templates(self, template_list):
        if self.template_root is not None:
            templates = []
            for root in self.template_root:
                templates += ["%s%s" % (root, template) for template in template_list]
            return templates
        return template_list

    def _filename(self, templates):
        template_name = templates[0]

        if self.template_root is not None:
            template_root = self.template_root[0]
            filename = template_name.split(template_root)[1]
        else:
            filename = template_name.split('/')[-1]

        return filename

    def _to_list(self, var):
        if isinstance(var, six.string_types):
            return [var]
        return var

    def write_template(self, template_list, filename=None, context=None, compress_type=None, optional=False):
        if context is None:
            context = {}

        template_list = self._to_list(template_list)
        templates_hierarchy = self._templates(template_list)

        try:
            render = render_to_string(templates_hierarchy, context)
        except TemplateDoesNotExist:
            if optional:
                return
            else:
                raise

        if filename is None:
            filename = self._filename(templates_hierarchy)

        if compress_type is not None:
            self.writestr(filename, render.encode('utf-8'), compress_type)
        else:
            self.writestr(filename, render.encode('utf-8'))

    def write_template_dir(self, directory, context=None, compress_type=None):

        for root, dirs, files in os.walk(directory):
            for f in files:
                template_name = root + "/" + f
                self.write_template(template_name, context=context, compress_type=compress_type)



#!/usr/bin/env python
import sys
from os import path
from django.conf import settings

PROJECT_DIR = path.dirname(path.realpath(__file__))


SETTINGS = {
    'DATABASES': {
        'default': {'ENGINE': 'django.db.backends.sqlite3'}
    },
    'INSTALLED_APPS': [
        'django.contrib.auth',
        'django.contrib.sessions',
        'django.contrib.contenttypes',
        'django.contrib.sites',
        'django_zipfile'
    ],
    'ROOT_URLCONF': 'tests.urls',
    'TEMPLATES': [
        {
            'BACKEND': 'django.template.backends.django.DjangoTemplates',
            'APP_DIRS': True,
            'DIRS': [
                path.join(PROJECT_DIR, 'templates'),
            ],
            'OPTIONS': {
                'context_processors': [
                    'django.contrib.auth.context_processors.auth',
                    'django.template.context_processors.debug',
                    'django.template.context_processors.i18n',
                    'django.template.context_processors.media',
                    'django.template.context_processors.static',
                    'django.template.context_processors.tz',
                    'django.contrib.messages.context_processors.messages',
                ],
            },
        },
    ],
    'MIDDLEWARE_CLASSES': (
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.middleware.common.CommonMiddleware',
        'django.middleware.csrf.CsrfViewMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
    ),
    'ALLOWED_HOSTS': ['*'],
    'SITE_ID': 1,
}


def runtests(*test_args):
    import django.test.utils
    settings.configure(**SETTINGS)
    django.setup()
    runner_class = django.test.utils.get_runner(settings)
    test_runner = runner_class(verbosity=1, interactive=True)
    failures = test_runner.run_tests(['tests'])
    sys.exit(failures)

if __name__ == '__main__':
    runtests()



# -*- encoding: utf-8
import unittest
try:
    from StringIO import StringIO
except ImportError:
    from io import BytesIO as StringIO
from django_zipfile import TemplateZipFile
from zipfile import ZIP_DEFLATED
from django.template.loader import TemplateDoesNotExist


class TestTemplateZipFile(unittest.TestCase):

    def setUp(self):
        self.fh = StringIO()
        self.zipfile1 = TemplateZipFile(self.fh, mode='w', compression=ZIP_DEFLATED, template_root=['override', 'default/'])
        self.zipfile2 = TemplateZipFile(self.fh, mode='w', compression=ZIP_DEFLATED, template_root='override/',)

    def test_names(self):
        templates = self.zipfile1._templates(['one.txt'])
        self.assertEqual(['override/one.txt', 'default/one.txt'], templates)

        filename = self.zipfile1._filename(templates)
        self.assertEqual('one.txt', filename)

        templates = self.zipfile1._templates(['two.txt', 'one.txt'])
        self.assertEqual(['override/two.txt', 'override/one.txt', 'default/two.txt', 'default/one.txt'], templates)

        templates = self.zipfile1._templates(['folderone/oneone.txt'])
        self.assertEqual(['override/folderone/oneone.txt', 'default/folderone/oneone.txt'], templates)

        filename = self.zipfile1._filename(templates)
        self.assertEqual('folderone/oneone.txt', filename)

        templates = self.zipfile1._templates(['folderone/onetwo.txt', 'folderone/oneone.txt'])
        self.assertEqual(['override/folderone/onetwo.txt', 'override/folderone/oneone.txt', 'default/folderone/onetwo.txt', 'default/folderone/oneone.txt'], templates)

        templates = self.zipfile2._templates(['one.txt'])
        self.assertEqual(['override/one.txt'], templates)

        filename = self.zipfile2._filename(templates)
        self.assertEqual('one.txt', filename)

        templates = self.zipfile2._templates(['two.txt', 'one.txt'])
        self.assertEqual(['override/two.txt', 'override/one.txt'], templates)

        templates = self.zipfile2._templates(['folderone/oneone.txt'])
        self.assertEqual(['override/folderone/oneone.txt'], templates)

        filename = self.zipfile2._filename(templates)
        self.assertEqual('folderone/oneone.txt', filename)

        templates = self.zipfile2._templates(['folderone/onetwo.txt', 'folderone/oneone.txt'])
        self.assertEqual(['override/folderone/onetwo.txt', 'override/folderone/oneone.txt'], templates)

        templates = self.zipfile2._templates([u'snowman_\u2603.txt'])
        filename = self.zipfile2._filename(templates)
        self.assertEqual(u'snowman_☃.txt', filename)

        templates = self.zipfile2._templates(['snowman_\xe2\x98\x83.txt'])
        filename = self.zipfile2._filename(templates)
        self.assertEqual('snowman_\xe2\x98\x83.txt', filename)

    def test_adding_files(self):
        self.assertRaises(TemplateDoesNotExist, self.zipfile1.write_template, 'abc.html')


if __name__ == '__main__':
    unittest.main()



import os
from setuptools import setup, find_packages

VERSION = "0.4.0"

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="django-zipfile",
    version=VERSION,
    description="A ZipFile subclass that accepts Django Templates",
    long_description=read('README.rst'),
    url='https://github.com/fcurella/django-zipfile',
    license='MIT',
    author='Flavio Curella',
    author_email='flavio.curella@gmail.com',
    packages=find_packages(exclude=['tests']),
    test_suite='tests.runtests.runtests',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
    install_requires=['six'],
)
